@@ -72,5 +72,6 @@ FC_OBJECT (COLOR,		FcTypeBool,	FcCompareBool)
 FC_OBJECT (SYMBOL,		FcTypeBool,	FcCompareBool)
 FC_OBJECT (FONT_VARIATIONS,	FcTypeString,	NULL)
 FC_OBJECT (VARIABLE,		FcTypeBool,	FcCompareBool)
-FC_OBJECT (FONT_HAS_HINT,	FcTypeBool,	NULL)
+FC_OBJECT (FONT_HAS_HINT,	FcTypeBool,	FcCompareBool)
+FC_OBJECT (ORDER,		FcTypeInteger,	FcCompareNumber)
 /* ^-------------- Add new objects here. */
